<template>
  <div class="example-data-table" ref="refView">
    <DataTable
      :height="height"
      :formObject="formObject"
      :buttons="buttons"
      :columns="columns"
      :data="data"
      :page="page"
      :limit="limit"
      :limits="limits"
      :total="total"
      :loading="loading"
      @on-selection-change="onSelectionChange"
      @on-change="onPageChange"
      @on-page-size-change="onPageSizeChange"
      @on-prev="onPrev"
      @on-next="onNext"
    ></DataTable>
    <DataTableInfo
      v-model="infoDialog.visible"
      :model="infoDialog.model"
      :dataId="infoDialog.dataId"
    ></DataTableInfo>
  </div>
</template>

<script>
import { ref, resolveComponent } from "vue";
import DataTable from "@/components/examples/data-table.vue";
import DataTableInfo from "./dialogs/info.vue";
export default {
  name: "ExampleDataTable",
  components: {
    DataTable,
    DataTableInfo,
  },
  data() {
    return {
      formObject: {
        options: {},
        items: [],
      },
      buttons: [
        {
          showName: "新增",
          onClick: () => this.onAddClick(),
        },
        {
          showName: "修改",
          onClick: () => this.onEditClick(),
        },
        {
          showName: "删除",
          onClick: () => this.onDelClick(),
        },
        {
          showName: "查看",
          onClick: () => this.onViewClick(),
        },
      ],
      columns: [],
      data: [],
      height: 0,
      // 分页信息
      page: 1,
      limit: 30,
      limits: [10, 20, 30],
      total: 0,
      selection: [],
      loading: false,
      // 查询表单定义
      form: {
        code: "",
        name: "",
        leader: "",
        tel: "",
        topCode: "",
        address: "",
        isThisLevel: false,
        isEscrow: false,
        isEnable: false,
      },
      // 弹框参数定义
      infoDialog: {
        model: "info",
        visible: false,
        dataId: "",
      },
    };
  },
  methods: {
    async loadForm() {
      this.formObject.options = {
        labelWidth: 110,
      };
      this.formObject.items = [
        {
          label: "仓库编码：",
          field: "SupplierCode",
          value: "",
          type: "input",
          span: 6,
        },
        {
          label: "仓库名称：",
          field: "SupplierName",
          value: "",
          type: "input",
          span: 6,
        },
        {
          label: "仓库电话：",
          field: "ASNCode",
          value: "",
          type: "input",
          span: 12,
        },
        {
          label: "地址：",
          field: "Invertory",
          value: "",
          type: "checkbox",
          span: 6,
        },
        {
          label: "是否本级",
          field: "PurchaseOrderNo",
          value: "",
          type: "checkbox",
          span: 6,
        },
        {
          label: "是否代管",
          field: "OrderDate",
          value: "",
          type: "checkbox",
          span: 12,
        },
      ];
    },
    async loadColumns() {
      this.columns = [
        {
          type: "selection",
          width: 40,
          align: "center",
        },
        {
          key: "index",
          title: " ",
          render: (h, { index }) => {
            return h("div", (this.page - 1) * this.limit + index + 1);
          },
          width: 40,
          align: "center",
        },

        {
          key: "code",
          title: "仓库编码",
          width: 150,
          sortable: true,
          resizable: true,
        },
        {
          key: "name",
          title: "仓库名称",
          width: 200,
          sortable: true,
          resizable: true,
        },
        {
          key: "leader",
          title: "仓库负责人",
          width: 150,
          sortable: true,
          resizable: true,
        },
        {
          key: "tel",
          title: "仓库电话",
          width: 150,
          sortable: true,
          resizable: true,
        },
        {
          key: "address",
          title: "地址",
          width: 150,
          sortable: true,
          resizable: true,
        },
        {
          key: "topCode",
          title: "上级仓库编码",
          width: 100,
          sortable: true,
          resizable: true,
        },
        {
          key: "isThisLevel",
          title: "是否本级",
          render: (h, { row }) => {
            return h(
              "div",
              h(resolveComponent("Checkbox"), {
                modelValue: row.isThisLevel,
                disabled: true,
                size: "small",
              })
            );
          },
          width: 80,
          align: "center",
          sortable: true,
        },
        {
          key: "isEscrow",
          title: "是否代管",
          render: (h, { row }) => {
            return h(
              "div",
              h(resolveComponent("Checkbox"), {
                modelValue: row.isEscrow,
                disabled: true,
                size: "small",
              })
            );
          },
          width: 80,
          align: "center",
          sortable: true,
        },
        {
          key: "isEnable",
          title: "是否启用",
          render: (h, { row }) => {
            return h(
              "div",
              h(resolveComponent("Checkbox"), {
                modelValue: row.isEnable,
                disabled: true,
                size: "small",
              })
            );
          },
          width: 80,
          align: "center",
          sortable: true,
        },
        {
          key: "row_button",
          title: " ",
          render: (h, { row, index }) => {
            return h(
              resolveComponent("Space"),
              {},
              {
                default: () => [
                  h("a", [
                    h(
                      "span",
                      {
                        style: {
                          marginRight: "12px",
                        },
                        onClick: () => this.onEditRowClick(index),
                      },
                      "修改"
                    ),
                    ,
                    h(
                      resolveComponent("Poptip"),
                      {
                        title: "是否确定删除此项？",
                        transfer: true,
                        confirm: true,
                        onOnOk: () => this.onDelClickItem(index),
                        onOnCancel: () => {},
                      },
                      {
                        default: () => h("span", "删除"),
                      }
                    ),
                  ]),
                ],
              }
            );
          },
          width: 100,
          align: "center",
          fixed: "right",
        },
      ];
    },
    // 加载数据行
    async loadData() {
      let msg = this.$Message.loading("正在加载数据...");
      this.loading = true;

      let { data, total } = await this.fakeDataList();

      msg();

      this.data = data;
      this.total = total;
      this.loading = false;
    },
    // 测试数据列表
    fakeDataList(limit) {
      return new Promise((resolve) => {
        let total = 99;
        let fakeList = [];
        for (let i = 0; i < (limit || this.limit); i++) {
          let row = {
            code: `仓库编码-${(this.page - 1) * this.limit + i + 1}`,
            name: `仓库名称-${(this.page - 1) * this.limit + i + 1}`,
            leader: `仓库负责人-${(this.page - 1) * this.limit + i + 1}`,
            tel: `仓库电话-${(this.page - 1) * this.limit + i + 1}`,
            address: `地址-${(this.page - 1) * this.limit + i + 1}`,
            topCode: `上级仓库编码-${(this.page - 1) * this.limit + i + 1}`,
            isThisLevel:
              ((this.page - 1) * this.limit + i + 1) % 2 == 0 ? true : false,
            isEscrow:
              ((this.page - 1) * this.limit + i + 1) % 2 == 0 ? true : false,
            isEnable:
              ((this.page - 1) * this.limit + i + 1) % 2 == 0 ? true : false,
          };
          if ((this.page - 1) * this.limit + i + 1 < 100) fakeList.push(row);
        }
        setTimeout(() => {
          resolve({ data: fakeList, total });
        }, 1 * 1000);
      });
    },
    resize() {
      this.height = this.refView.clientHeight - 20;
    },
    onAddClick() {
      this.infoDialog.model = "add";
      this.infoDialog.dataId = "";
      this.infoDialog.visible = true;
    },
    onEditClick() {
      if (this.selection.length == 0) return this.showWarning("未选择操作项");
      else if (this.selection.length > 1)
        return this.showWarning("只能选择一项进行编辑");
      let code = this.selection[0].code;
      this.infoDialog.model = "edit";
      this.infoDialog.dataId = code;
      this.infoDialog.visible = true;
    },
    onEditRowClick(index) {
      let code = this.data[index].code;
      this.infoDialog.model = "edit";
      this.infoDialog.dataId = code;
      this.infoDialog.visible = true;
    },
    onViewClick() {
      if (this.selection.length == 0) return this.showWarning("未选择操作项");
      else if (this.selection.length > 1)
        return this.showWarning("只能选择一项查看");
      let code = this.selection[0].code;
      this.infoDialog.model = "info";
      this.infoDialog.dataId = code;
      this.infoDialog.visible = true;
    },
    showWarning(tip) {
      this.$Message.warning({
        content: tip,
        duration: 0,
        closable: true,
      });
    },
    async onDelClick() {
      this.$Modal.confirm({
        title: "提示",
        content: "是否确认删除这些条目",
        loading: true,
        onOk: () => {
          setTimeout(() => {
            this.$Modal.remove();
            this.$Message.success("删除成功！");
          }, 2000);
        },
      });
    },
    async onDelClickItem(index) {
      const msg = this.$Message.loading({
        content: "删除中...",
        duration: 0,
      });
      setTimeout(() => {
        this.data.splice(index, 1);
        msg();
        this.$Message.success("删除成功！");
      }, 2000);
    },
    onSelectionChange(selection) {
      this.selection = selection;
    },
    onPageChange(page) {
      this.page = page;
      this.loadData();
    },
    onPageSizeChange(limit) {
      this.limit = limit;
      this.loadData();
    },
    onPrev(page) {
      this.page = page;
      this.loadData();
    },
    onNext(page) {
      this.page = page;
      this.loadData();
    },
  },
  async mounted() {
    await this.loadForm();
    await this.loadColumns();
    this.$nextTick(() => {
      this.resize();
      this.loadData();
    });
  },
  setup() {
    const refView = ref(null);
    return { refView };
  },
};
</script>

<style lang="less">
.example-data-table {
  height: 100%;
  .search-form {
    margin-bottom: 9px;
  }
  .table {
    height: calc(100% - 170px);
  }
  .ivu-form .ivu-form-item-label,
  .ivu-checkbox-wrapper {
    font-size: 12px;
  }
  .ivu-input-wrapper {
    .ivu-icon {
      cursor: pointer;
    }
  }
  .ivu-checkbox-input[disabled] {
    cursor: default;
  }
  .ivu-checkbox-disabled .ivu-checkbox-inner {
    border-color: #666;
    background-color: #fff;
  }
  .ivu-checkbox-disabled.ivu-checkbox-checked .ivu-checkbox-inner:after {
    border-color: #666;
  }
}
</style>